﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.DataAnalysis {

  /// <summary>
  /// this is not a proper distance 
  /// </summary>
  [StorableClass]
  [Item("InnerProductDistance", "The Angluar Distance as defined as a normalized distance measure dependent on the angle between two vectors.\nIt is designed for vectors with all positive coordinates")]
  public class InnerProductDistance : DistanceBase<IReadOnlyList<double>> {

    #region HLConstructors
    [StorableConstructor]
    protected InnerProductDistance(bool deserializing) : base(deserializing) { }
    protected InnerProductDistance(InnerProductDistance original, Cloner cloner)
      : base(original, cloner) { }
    public InnerProductDistance() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new InnerProductDistance(this, cloner);
    }
    #endregion

    #region statics
    public static double GetDistance(IReadOnlyList<double> point1, IReadOnlyList<double> point2) {
      if (point1.Count != point2.Count) throw new ArgumentException("Inner Product distance not defined on vectors of different length");
      return point1.Zip(point2, (x, y) => x * y).Sum();
    }
    public static double GetDistance(IEnumerable<double> a, IEnumerable<double> b, double threshold) {
      return GetDistance(a.ToArray(), b.ToArray());     //no shortcut evaluation for Inner Product (summands may be negative => no way of telling if threshold is reached or not)
    }
    #endregion
    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b) {
      return GetDistance(a, b);
    }
    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b, double threshold) {
      return GetDistance(a, b, threshold);
    }
  }
}
